// var config = require('../config.js');
import MtcHandler from "./mtc";
let eventDesc = {
    301: { desc: "ACTIVITY_PAUSE" },
    302: { desc: "ACTIVITY_RESUME" },
    321: { desc: "DISABLE_LOCAL_VIDEO" },//关闭摄像头
    322: { desc: "ENABLE_LOCAL_VIDEO" },//开启摄像头
    323: { desc: "DISABLE_LOCAL_AUDIO" },//关闭麦克风
    324: { desc: "ENABLE_LOCAL_AUDIO" },//开启麦克风
    329: { desc: "CHANGE_TO_EARPIECE" },//转听筒
    330: { desc: "CHANGE_TO_SPEAKER_PHONE" },//转外放
    363: { desc: "ERROR_CAMERA_PERMISSION" },//相机权限错误	
    364: { desc: "ERROR_MIC_PERMISSION" },//mic权限错误	
    366: { desc: "ERROR_OPEN_CAMERA" },// 打开摄像头错误-1007	
    367: { desc: "ERROR_OPEN_MIC" },// 打开麦克风错误/麦克风采集异常
    503: { desc: "VIDEO_FIRST_FRAME", extra: true },// 订阅到视频首帧

}
export default class Chat {

    //构造器
    constructor(app) {
        this.app = app; // 方便在Chat内部操作app
        this.meeting_sdk_version = '1.1.0'; //sdk版本号
        this.meeting_protocol_version = '2.0.0'; //服务端版本号
        this.feedIds = [];
        this.requestId = 1000; //请求Id
        this.sessionId = null; //会话sessionId
        this.connectionId = null; //连接id
        this.systemInfo = null; //系统信息
        this.room_status = 'close'; //房间状态
        this.msg_text_id = 0;
        this.send_msg_map = new Map();;
        this.pushUrl = '';
        this.roomId = ''; //房间号
        this.publishStreamId = ''; //发布streamId
        this.receive_msg_array = [];
        this.room_heartbeat_timer_id = -1;
        this.msg_recv_id_map_ = new Map();
        this.feedId_streamId = new Map();

        this.mtc = null;
        this.mtc_state = "close";

        //业务回调方法
        this.OnError = null;
        this.OnCreateRoom = null;
        this.OnJoinRoom = null;
        this.OnPublish = null;
        this.OnSubscribe = null;
        this.OnGetSign = null;
        this.OnLeaveRoom = null;
        this.OnConnect = null;
        this.OnDisConnect = null;
        this.OnParticipantLeaveRoom = null;
        this.receive_msgIds = [];
        
    }

    /* mtc相关  */
    /* mtc相关  */
    /* mtc相关  */
    /* 初始化连接 */
    onMtcOpen() {
        this.warning("~~~~~ artvc MtcOpen WebSocket连接已打开");
        // 变更状态
        this.mtc_state = "running";
        let data = {}
        data.code = 0;
        data.msg = "SUCCESS";
        this.innerOnConnect(data);
    }

    onMtcReOpen() {
        this.warning("~~~~~ artvc MtcReOpen");
        // 变更状态
        this.mtc_state = "running";
        if (this.room_status == "working") {
            this.heartbeat();
        }
    }

    onMtcClose() {
        //关闭SDK
        this.warning(`~~~~~~~~ artvc MTC close!`);
        this.leaveType = 1;
        //变更状态
        this.mtc_state = "close";
        //关闭SDK
        this.Disconnect.bind(this);
    }

    onMtcError(code, msg) {
        this.warning(`~~~~~~~~ artvc MTC error! code: ${code}, msg: ${msg}`);
        // 初始化失败回调
        if (this.mtc_state == "init") {
            this.leaveType = 2;
            this.mtc_state = "init_fail";
            this.OnInnerConnectFailed(-102, `${code}_${msg}`);
        } else if (this.mtc_state == "close") {
            // DO NOTHING
        } else {
            this.mtc.reconnect();
        }
    }

    //收到消息
    onMtcMessage(res) {
        //房间已关闭
        let msg = JSON.parse(res);
        let requestId = msg.requestId;
        let opcmd = msg.opcmd;
        if (this.room_status == "close" && opcmd != 1238 && opcmd != 1028) {
            this.warning(`artvc onMtcMessage room status is ${this.room_status}`);
            return;
        }
        
        if (this.roomId) {
            this.trace(`S->C: [wx onReceiveMessage] roomId = ${this.roomId}, requestId = ${requestId}, msg =${res}`);
        } else {
            this.trace(`S->C: [wx onReceiveMessage] requestId = ${requestId}, msg =${res}`);
        }
        if ((opcmd >= 1250 && opcmd <= 1256) || (opcmd >=1502 && opcmd <= 1600)) {
            // room push, 回ack
            this.sendAck(requestId);
        }
       
        //校验session
        if (msg.sessionId && this.sessionId && this.sessionId != msg.sessionId) {
            this.warning(` push msg: wrong sessionId. msg.sessionId=${msg.sessionId}, this.sessionId=${this.sessionId}`);
            return;
        }
        //未收到session_token则先不处理回调数据
        if (msg.opcmd != 1042 && msg.opcmd != 1044 && !this.sessionId && msg.sessionId && this.receive_msg_array) {
            this.trace("sessionId not found, Postprocessing.");
            this.receive_msg_array.push(res);
            return;
        }

        // 缓存已收到消息，去重
        if (this.msg_recv_id_map_.get(requestId) == 1) {
            this.trace(`recv repeat msg, requestId = ${requestId}`);
            return;
        }
        this.msg_recv_id_map_.set(requestId, 1);
        this.getSocketMsg(msg); // 收到的消息为字符串，需处理一下
    }

    OnInnerConnectFailed(code, msg) {
        this.Disconnect();
        this.OnError(code, msg);
    }

    //发送消息
    sendMsg(requestData) {
        this.mtc.send(requestData, true);
    };

    /* 回调通知业务 */
    /* 回调通知业务 */
    /* 回调通知业务 */
    innerOnError(data) {
        this.OnError && this.OnError(data);
    }

    innerOnCreateRoom(data) {
        this.OnCreateRoom && this.OnCreateRoom(data);
    };

    innerOnJoinRoom(data) {
        this.OnJoinRoom && this.OnJoinRoom(data);
    };
    innerOnClientJoin(data) {
        this.OnClientJoin && this.OnClientJoin(data);
    }
    innerOnPublish(data) {
        this.OnPublish && this.OnPublish(data);
    };
    innerOnSubscribe(data) {
        this.OnSubscribe && this.OnSubscribe(data);
    };
    innerOnSendTxtMessageSucc(msgId) {
        this.OnSendTxtMessageSucc && this.OnSendTxtMessageSucc(msgId);
    };
    innerOnInviteSucc(res) {
        this.OnInviteSucc && this.OnInviteSucc();
    };
    innerOnInviteFail(res) {
        this.OnInviteFail && this.OnInviteFail(res.code, res.msg);
    };
    innerOnReceiveInvite(res) {
        if (this.sessionId) {
            this.warning("invite push msg: user is working!!!");
            this.ReplyInviteToJoinRoom(res.roomId, res.inviter, 3, res.inviteId, res.inviteInfo);
            return;
        }
        this.OnInviteRequest && this.OnInviteRequest(res.roomId, res.rtoken, res.inviter, res.extra, res.inviteId, res.inviteInfo);
    };
    innerOnReplyInviteRes(res){
        if (res.code == 0) {
            this.OnReplyInviteOK && this.OnReplyInviteOK();
        } else {
            this.OnReplyInviteFail && this.OnReplyInviteFail(res.code, res.msg);
        }
    };
    innerOnGetTxtMessage(data) {
        this.OnGetTxtMessage && this.OnGetTxtMessage(data);
    }
    //取消发布
    innerOnUnPublish(feedId) {
        this.OnUnPublish && this.OnUnPublish(feedId);
    }
    //取消订阅
    innerOnUnsubscribe(res) {
        let streamId = res.streamId;
        this.OnUnsubscribe && this.OnUnsubscribe(streamId);
    }
    innerOnLeaveRoom(data) {
        this.OnLeaveRoom && this.OnLeaveRoom(data);
    };

    innerOnConnect(data) {
        this.OnConnect && this.OnConnect(data);
    };

    innerOnDisConnect(data) {
        this.OnDisConnect && this.OnDisConnect(data);
    };

    innerOnParticipantLeaveRoom(data) {
        this.OnParticipantLeaveRoom && this.OnParticipantLeaveRoom(data);
    };

    onMtcReqSignInfo(bizName, subBiz, uid, workspaceId) {
        return this.OnGetSign && this.OnGetSign(bizName, subBiz, uid, workspaceId);
    }

    // 收到消息 回调消息
    getSocketMsg(responData) {
        var data = {};
        let opcmd = responData.opcmd;
        //创建房间response
        if (opcmd == 1042) {
            if (responData.code == 0) {
                const sessionId = responData.sessionId;
                const connectionId = responData.connectionId;
                if (sessionId) {
                    this.room_status == 'working';
                    clearTimeout(this.room_create_Room_timer_id);
                    this.sessionId = sessionId;
                    this.connectionId = connectionId;
                    this.roomId = responData.roomId;
                    this.rtoken = responData.rtoken;
                    data.roomId = this.roomId;
                    data.rtoken = this.rtoken;
                    this.time = responData.time;
                    //解析rtmp地址
                    this.rtmpUrl = responData.gw_server;
                    const heartBeatTime = responData.config.heartbeatInterval * 1000;
                    // 启动房间心跳
                    this.room_heartbeat_timer_id = setInterval(() => {
                        this.heartbeat();
                    }, heartBeatTime);
                    // 给每个订阅者发消息
                    data.code = 0;
                    data.code = '创建房间成功';
                    this.innerOnCreateRoom(data);

                }
                // 执行未缓存的消息
                if (this.receive_msg_array.length > 0) {
                    for (let i = 0; i < this.receive_msg_array.length; i++) {
                        this.onMtcMessage(this.receive_msg_array[i]);
                        delete this.receive_msg_array[i];
                    }
                }
            } else {
                this.receive_msg_array = [];
                // 创建房间失败回调
                data.code = -10412;
                data.msg = `创建房间失败`;
                this.innerOnError(data);
                // 退出房间
                // this.closeRoom(false);
            }

            //加入房间response
        } else if (opcmd == 1044) {
            const sessionId = responData.sessionId;
            //判断房间状态
            if (responData.code == 0) {
                if (this.room_status == 'joining' && sessionId) {
                    this.room_status == 'working';
                    // 执行未缓存的消息
                    if (this.receive_msg_array.length > 0) {
                        for (let i = 0; i < this.receive_msg_array.length; i++) {
                            this.onReceiveMessage(this.receive_msg_array[i]);
                            delete this.receive_msg_array[i];
                        }
                    }
                    clearTimeout(this.room_joinRoom_timer_id);
                    this.time = responData.time;
                    //解析rtmp地址
                    this.rtmpUrl = responData.gw_server;
                    data.code = 0;
                    data.msg = '加入房间成功';
                    this.joinPushMsg(sessionId, responData, data);
                }
            } else {
                this.receive_msg_array = [];
                data.code = -10433;
                data.msg = `加入房间失败`;
                this.room_status = 'close';
                clearTimeout(this.room_joinRoom_timer_id);//不重复提示加入超时
                this.innerOnError(data);
                // 退出房间
                // this.closeRoom(false);
            }

            //发布response
        } else if (opcmd == 1046) {
            let code = responData.code;
            if (code == 0) {
                this.publishStreamId = responData.streamId;
                let url = `rtmp://${this.rtmpUrl}/rtc/push/${this.publishStreamId}?roomId=${this.roomId}&userId=${this.uid_}&token=${this.rtoken}`;
                this.trace(`publish streamId=${this.publishStreamId}, ${url}`);
                data.pushUrl = url;
                this.pushUrl = url;
                // 给每个订阅者发消息
                this.innerOnPublish(data);
            } else {
                data.code = -10451;
                data.msg = `发布失败`;
                this.innerOnError(data);
            }

            //订阅response
        } else if (opcmd == 1048) {
            let code = responData.code;
            if (code == 0) {
                let streamId = responData.streamId;
                let url = `rtmp://${this.rtmpUrl}/rtc/pull/${streamId}?roomId=${this.roomId}&userId=${this.uid_}&token=${this.rtoken}`;
                let feedId = responData.feed.feedId[0];
                this.trace(`subscribe streamId=${streamId}, feedId=${feedId}, ${url}`);
                data.liveUrl = url;
                data.feedId = feedId;
                data.streamId = responData.streamId
                this.feedId_streamId.set(feedId, streamId)
                // 给每个订阅者发消息
                this.innerOnSubscribe(data);
            } else {
                data.code = -10471;
                data.msg = '订阅失败';
                this.innerOnError(data);
            }

            //push房间与会者信息
        } else if (opcmd == 1250) {
            this.innerOnGetFeedIds(responData.participants);

            //push新发布
        } else if (opcmd == 1252) {
            let feedId = responData.feed.feedId;
            let uid = responData.feed.uid;
            let tag = responData.feed.tag;
            this.feedIds.push({ feedId, uid });
            this.innerOnClientJoin({ feedId, uid, tag });

            //response离开房间
        } else if (opcmd == 1050) {
            this.trace("离开房间成功");

            //response取消发布
        } else if (opcmd == 1052) {
            this.trace("取消发布成功");

            //response取消订阅
        } else if (opcmd == 1054) {
            if(responData.code == 0) {
                this.trace("取消订阅成功");
                this.innerOnUnsubscribe(responData);
            } else {
                this.trace("取消订阅失败");
                data.code = -10481;
                data.msg = `取消订阅失败`;
                this.innerOnError(data)
            }
            //response心跳
        } else if (opcmd == 1056) {
            let code = responData.code;
            if (code != 0) {
                this.trace("心跳响应失败");
                this.LeaveRoom();
                this.Disconnect();
            }

            //push取消发布
        } else if (opcmd == 1254) {
            this.trace('push取消发布');
            let feed = responData.feed;
            let streamId = this.feedId_streamId.get(feed.feedId)
            if(streamId) {
                this.feedId_streamId.delete(feed.feedId);
                this.UnSubscribe(streamId);
                this.innerOnUnPublish(feed);
            }
            //push退出房间
        } else if (opcmd == 1256) {
            let uid = responData.participant
            data.participant = uid;
            for (let i = this.feedIds.length - 1; i >= 0; i--) {
                let item = this.feedIds[i];
                if (uid == item.uid) {
                    this.feedIds.splice(i, 1)
                    data.streamId = item.feedId
                    let streamId = this.feedId_streamId.get(item.feedId)
                    if(streamId) {
                        this.feedId_streamId.delete(item.feedId);
                        this.UnSubscribe(streamId);
                    }
                }
            }
            data.exitType = responData.exitType;
            this.innerOnParticipantLeaveRoom(data);

            // response发送文本
        } else if (opcmd == 1402) {
            if (responData.code == 0) {
                this.trace(`S->C: [artvc onReceiveMessage] send msg response msg: ${JSON.stringify(responData)}`);
                let msgId = this.send_msg_map.get(responData.requestId);
                this.send_msg_map.delete(responData.requestId);
                this.innerOnSendTxtMessageSucc(msgId);
            } else {
                this.innerOnError(responData);
            }

            //push文本消息
        } else if (opcmd == 1502) {
            if (this.receive_msgIds.includes(responData.msgId)) {
                this.trace(`recv duplicate text msg: ${responData.msgId}`);
                return;
            }
            this.receive_msgIds.push(responData.msgId)
            this.innerOnGetTxtMessage(responData)

            //response事件上报
        } else if (opcmd == 1404) {
            if (responData.code == 0) {
                this.trace(`send client event success! ${JSON.stringify(responData.msg)}`);
            } else {
                this.warning(`send client event failed! ${JSON.stringify(responData.msg)}`);
            }
        } else if ( opcmd == 1026) {
            if (responData.code == 0) {
                this.innerOnInviteSucc(responData);
            } else {
                this.innerOnInviteFail(responData);
            }
        } else if (opcmd == 1238) {
            this.innerOnReceiveInvite(responData);
        } else if(opcmd == 1028) {
            this.innerOnReplyInviteRes(responData);
        }

    }

    //push加入房间信息
    joinPushMsg(sessionId, responData, data) {
        this.sessionId = sessionId;
        this.connectionId = responData.connectionId;
        const heartbeatInterval = (responData.config.heartbeatInterval * 1000) / 3;
        // 启动房间心跳
        this.room_heartbeat_timer_id = setInterval(() => {
            this.heartbeat();
        }, heartbeatInterval);
        // 给每个订阅者发消息
        this.innerOnJoinRoom(data);
    }

    //获取feedId
    innerOnGetFeedIds(participants) {
        for (let i = 0; i < participants.length; i++) {
            let part = participants[i];
            let publishs = part.publish;
            let uid = part.uid;
            //订阅
            for (let j = 0; j < publishs.length; j++) {
                let publish = publishs[j];
                let feedId = publish.feedId;
                let tag = publish.tag;
                this.feedIds.push({ feedId, uid,tag });
            }
        }
        this.OnGetFeedIds && this.OnGetFeedIds(this.feedIds.slice(0))
    }

    //回去ack
    sendAck(ackRequestId) {
        // 组装协议
        let req_json = {};
        req_json['opcmd'] = 1019;
        req_json['requestId'] = this.getRquestId();
        req_json['sessionId'] = this.sessionId;
        if (this.connectionId) {
            req_json['connectionId'] = this.connectionId;
        }
        req_json['ackRequestId'] = ackRequestId;
        const requestData = JSON.stringify(req_json);
        this.trace(`C-->S sendAck requestInfo = ${requestData}`);
        this.sendMsg(requestData);
    }

    //发心跳
    heartbeat() {
        let req_json = {};
        req_json['opcmd'] = 1055;
        req_json['requestId'] = this.getRquestId();
        req_json['sessionId'] = this.sessionId;
        if (this.connectionId) {
            req_json['connectionId'] = this.connectionId;
        }
        const requestData = JSON.stringify(req_json);
        this.trace(`C-->S heartbeat requestInfo = ${requestData}`);
        this.sendMsg(requestData);
    }



    //房间相关
    //房间相关
    //房间相关
    //初始化房间
    initRoom(config) {
        this.trace(`artvc initRoom config = ${JSON.stringify(config)}`);
        this.uid_ = config.uid;
        this.biz_name_ = config.biz_name;
        this.sub_biz = config.appId;
        this.room_server_url_ = config.server_url;
        this.workspaceId = config.workspaceId ? config.workspaceId : "default";
        this.network_check_timeout = config.network_check_timeout;
    }

    //请求离开房间
    innerLeaveRoom() {
        if (this.room_status != 'close') {
            let req_json = {};
            req_json['opcmd'] = 1049;
            req_json['requestId'] = this.getRquestId();
            req_json['sessionId'] = this.sessionId;
            if (this.connectionId) {
                req_json['connectionId'] = this.connectionId;
            }
            const requestData = JSON.stringify(req_json);
            this.trace(`C-->S innerLeaveRoom requestInfo = ${requestData}`);
            this.sendMsg(requestData);
        }
    }

    //重置room信息
    resetRoom() {
        this.sessionId = null; //会话sessionId
        this.connectionId = null; //连接id
        this.roomId = ''; //房间号
        this.publishStreamId = ''; //发布streamId
        this.feedIds = [];
        this.room_status = 'close'; //房间状态
        this.pushUrl = '';
        //缓存已收到消息,去重
        if (this.msg_recv_id_map_) {
            this.msg_recv_id_map_.clear();
        }
        if (this.feedId_streamId) {
            this.feedId_streamId.clear();
        }
    }

    //退出房间
    closeRoom(leaveRoomType) {
        this.warning(`closeRoom leaveRoomType = ${leaveRoomType}`);
        if (this.room_status != "close") {
            // 停止房间心跳
            if (this.room_heartbeat_timer_id != -1) {
                this.trace(`close room: close room heartbeat timer, timer_id = ${this.room_heartbeat_timer_id}`);
                clearInterval(this.room_heartbeat_timer_id);
                this.room_heartbeat_timer_id = -1;
            }
            // 关闭音视频
            // this.clearAllStream(5009);
            // this.clearUserInfo();
            if (leaveRoomType) {
                // 发送离开房间请求
                this.trace(`closeRoom request leaveRoomType=${leaveRoomType}`);
                this.innerLeaveRoom();
            }
            //回调
            let leaveType = 2;
            if (leaveRoomType && (this.leaveType == 1 || typeof (this.leaveType) == "undefined")) {
                leaveType = 1;
            }
            let data = {};
            data.leaveType = leaveType;
            this.innerOnLeaveRoom(data);
            //### ResetRoom 重置房间状态
            this.resetRoom();
        }
    };

    /* 主调接口 */
    /* 主调接口 */
    /* 主调接口 */
    //建立连接
    Connect(config) {
        if (this.mtc_state == "init" || this.mtc_state == "running") {
            this.trace("SDK初始化已完成");
            return -1;
        }
        this.initRoom(config);
        this.mtc = new MtcHandler();
        this.mtc.OnMtcOpen = this.onMtcOpen.bind(this);
        this.mtc.OnMtcReOpen = this.onMtcReOpen.bind(this);
        this.mtc.OnMtcClose = this.onMtcClose.bind(this);
        this.mtc.OnMtcError = this.onMtcError.bind(this);
        this.mtc.OnMtcReqSignInfo = this.onMtcReqSignInfo.bind(this);
        this.mtc.OnMtcMessage = this.onMtcMessage.bind(this);
        this.mtc_state = "init";
        let mtc_config_param = {};
        mtc_config_param.uid = this.uid_;
        mtc_config_param.biz_name = this.biz_name_;
        mtc_config_param.sub_biz = this.sub_biz;
        if (this.workspaceId) {
            mtc_config_param.workspaceId = this.workspaceId;
        }
        mtc_config_param.mtc_server_url = this.room_server_url_;
        mtc_config_param.debug = true;
        mtc_config_param.network_check_timeout = this.network_check_timeout;
        this.mtc_timeout_timer_id = setTimeout(function () {
            if (this.mtc_state == "init") {
                let data = {};
                data.code = -101;
                data.msg = "建立连接超时";
                this.innerOnError(data);
            }
        }.bind(this), 5000);
        this.mtc.init(mtc_config_param);
    }

    Disconnect() {
        // 关闭房间
        this.trace("CloseConnect");
        if (this.mtc_state != "close") {
            this.mtc_state = "close";
            this.mtc.close();
        } else {
            this.mtc = null;
        }
        if (this.mtc_timeout_timer_id != -1) {
            this.trace(`close mtc timer, timer_id = ${this.mtc_timeout_timer_id}`);
            clearTimeout(this.mtc_timeout_timer_id);
            this.mtc_timeout_timer_id = -1;
        }
        // 清空用户信息
        this.sub_biz = null;
        this.biz_name_ = null;
        this.uid_ = null;
        this.room_server_url_ = null;
        this.network_check_timeout = 0;
    }

    //创建房间
    CreateRoom() {
        if (this.room_status == 'creating') {
            this.trace('正在创建房间,请不要重复点击');
            return
        }
        if (this.room_status != 'close') {
            this.trace('已经在房间中');
            return
        }
        let req_json = {};
        req_json['opcmd'] = 1041;
        req_json['requestId'] = this.getRquestId();
        req_json['uid'] = this.uid_;
        req_json['bizName'] = this.biz_name_;
        req_json['subBiz'] = this.sub_biz;
        req_json['sdkVersion'] = this.meeting_protocol_version;
        req_json['type'] = 2; // 2：RTC模式;3：直播模式
        req_json['sign'] = this.onMtcReqSignInfo(this.biz_name_, this.sub_biz, this.uid_, this.workspaceId);;
        let extra = {};
        this.getARTVCUserInfo(extra);
        extra.workspaceId = this.workspaceId;
        req_json['extra'] = extra;
        this.room_status = 'creating';
        this.room_create_Room_timer_id = setTimeout(() => {
            if (!this.sessionId) {
                this.receive_msg_array = [];
                this.room_status = 'close';
                clearTimeout(this.room_create_Room_timer_id);
                let data = {};
                data.code = -10411;
                data.msg = '创建房间超时';
                this.innerOnError(data);
            }
        }, 5000);
        const requestData = JSON.stringify(req_json);
        this.trace(`C-->S CreateRoom requestInfo = ${requestData}`);
        this.sendMsg(requestData);
    }

    //加入房间
    JoinRoom(config) {
        let roomId = config.roomId;
        let rtoken = config.rtoken;
        this.roomId = roomId;
        this.rtoken = rtoken;

        if (this.room_status == 'joining') {
            this.trace('正在加入房间,请不要重复点击');
            return
        }
        if (this.room_status != 'close') {
            this.trace('已经在房间中');
            return
        }

        let req_json = {};
        req_json['opcmd'] = 1043;
        req_json['requestId'] = this.getRquestId();
        req_json['bizName'] = this.biz_name_;
        req_json['subBiz'] = this.sub_biz;
        req_json['uid'] = this.uid_;
        req_json['sdkVersion'] = this.meeting_protocol_version;
        req_json['sign'] = this.onMtcReqSignInfo(this.biz_name_, this.sub_biz, this.uid_, this.workspaceId);
        req_json['rtoken'] = rtoken;
        req_json['roomId'] = roomId;
        req_json['engine'] = 0;
        let extra = {};
        this.getARTVCUserInfo(extra);
        extra.workspaceId = this.workspaceId;
        req_json['extra'] = extra;
        this.room_status = 'joining';
        this.room_joinRoom_timer_id = setTimeout(() => {
            if (!this.sessionId) {
                this.receive_msg_array = []; //需要清空数组
                this.room_status = 'close';
                clearTimeout(this.room_joinRoom_timer_id);
                let data = {};
                data.code = -10432;
                data.msg = '加入房间超时';
                this.innerOnError(data);
            }
        }, 5000);
        const requestData = JSON.stringify(req_json);
        this.trace(`C-->S JoinRoom requestInfo = ${requestData}`);
        this.sendMsg(requestData);
    }

    //发布
    Publish(config) {
        let resolution = config.resolution;
        let fps = parseInt(config.fps);
        let maxBitrate = parseInt(config.maxBitrate);
        let req_json = {};
        req_json['opcmd'] = 1045;
        req_json['requestId'] = this.getRquestId();
        req_json['sessionId'] = this.sessionId;
        if (this.connectionId) {
            req_json['connectionId'] = this.connectionId;
        }
        req_json['mediaSource'] = "VIDEO_SOURCE_CAMERA";
        req_json['tag'] = config.tag ? config.tag : req_json['mediaSource'];

        let videoProfile = {};
        videoProfile.fps = fps;
        videoProfile.maxBitrate = maxBitrate;
        videoProfile.resolution = resolution;
        let profile = {};
        profile.videoProfile = videoProfile;
        profile.videoEnable = true;
        profile.audioEnable = true;
        req_json['profile'] = profile;
        const requestData = JSON.stringify(req_json);
        this.trace(`C-->S Publish requestInfo = ${requestData}`);
        this.sendMsg(requestData);
    }

    //取消发布
    UnPublish() {
        if (this.room_status != 'close') {
            let req_json = {};
            req_json['opcmd'] = 1051;
            req_json['requestId'] = this.getRquestId();
            req_json['sessionId'] = this.sessionId;
            if (this.connectionId) {
                req_json['connectionId'] = this.connectionId;
            }
            req_json['streamId'] = this.publishStreamId;
            const requestData = JSON.stringify(req_json);
            this.trace(`C-->S UnPublish requestInfo = ${requestData}`);
            this.sendMsg(requestData);
        } else {
            this.trace(`UnPublish room_status = ${this.room_status}`);
        }
    };

    //订阅
    Subscribe(feedId) {
        let hasFeedId = this.feedIds.some(function (item) {
            return item.feedId == feedId
        });
        if (!hasFeedId) {
            this.warning(`${feedId} did not exist`);
            return
        }
        if (feedId) {
            this.trace(`Subscribe feedId = ${feedId}`);
            let req_json = {};
            req_json['opcmd'] = 1047;
            req_json['requestId'] = this.getRquestId();
            req_json['sessionId'] = this.sessionId;
            if (this.connectionId) {
                req_json['connectionId'] = this.connectionId;
            }
            let feed_json = {};
            let feedIds = [];
            feedIds.push(feedId);
            feed_json['feedId'] = feedIds;
            feed_json['strategy'] = 1;
            feed_json['videoEnable'] = true;
            feed_json['audioEnable'] = true;
            req_json['feed'] = feed_json; //订阅的流
            const requestData = JSON.stringify(req_json);
            this.trace(`C-->S Subscribe requestInfo = ${requestData}`);
            this.sendMsg(requestData);
        }
    }

    //取消订阅
    UnSubscribe(streamId) {
        if (this.room_status != 'close') {
            //取消订阅媒体流
            let req_json = {};
            req_json['opcmd'] = 1053;
            req_json['requestId'] = this.getRquestId();
            req_json['sessionId'] = this.sessionId;
            if (this.connectionId) {
                req_json['connectionId'] = this.connectionId;
            }
            req_json['streamId'] = streamId;
            const requestData = JSON.stringify(req_json);
            this.trace(`C-->S UnSubscribe requestInfo = ${requestData}`);
            this.sendMsg(requestData);
        } else {
            this.trace(`UnSubscribe room_status = ${this.room_status}`);
        }
    }
    //发送文本信息
    SendTxtMessage(config_param) {
        let req_json = {};
        let requestId = this.getRquestId();
        let msgId = ++this.msg_text_id;
        msgId = msgId + (new Date()).valueOf();
        req_json['opcmd'] = 1401;
        req_json['requestId'] = requestId;
        req_json['sessionId'] = this.sessionId;
        if (this.connectionId) {
            req_json['connectionId'] = this.connectionId;
        }
        req_json['uid'] = this.uid_;
        req_json['peer'] = config_param.uids;
        req_json['msg'] = config_param.msg;
        req_json['timestamp'] = config_param.timestamp;
        req_json['msgId'] = msgId;
        this.send_msg_map.set(requestId, this.msg_text_id);
        const requestData = JSON.stringify(req_json);
        this.trace(`C-->S SendTxtMessage requestInfo = ${requestData}`);
        this.sendMsg(requestData);
    }
    SendInvite (invitee) {
        let req_json = {};
        req_json['opcmd'] = 1025;
        req_json['requestId'] = this.getRquestId();
        req_json['sessionId'] = this.sessionId;
        req_json['invitee'] = invitee;
        req_json['inviteId'] = (new Date()).valueOf() + "";
        let inviteInfo = {
            audioEnable: 1,
            videoEnable: 1
        };
        req_json['inviteInfo'] = inviteInfo;
        req_json['channelType'] = 0;
        let extra = {};
        extra.url = "";
        req_json['extra'] = extra;
        let requestData = JSON.stringify(req_json);
        this.trace("C->S: invite request= " + requestData);
        this.sendMsg(requestData);
      }
     // 答复(接受/拒绝)邀请加入房间
    ReplyInviteToJoinRoom = function (room_id, inviter, reply, inviteId, inviteInfo, extra) {
        let req_json = {};
        req_json['opcmd'] = 1027;
        req_json['requestId'] = this.getRquestId();
        req_json['bizName'] = this.biz_name_;
        req_json['subBiz'] = this.sub_biz;
        req_json['uid'] = this.uid_;
        req_json['inviteId'] = inviteId;
        req_json['roomId'] = room_id;
        req_json['inviter'] = inviter;
        req_json['reply'] = reply;
        req_json['inviteInfo'] = inviteInfo;
        req_json['extra'] = extra;

        // 扩展字段（业务用）
        //mpaas工作空间（mpaas专用）
        if (this.workspaceId) {
        //mpaas工作空间（mpaas专用）
        let innerExtra = {};
        innerExtra.workspaceId = this.workspaceId;
        req_json['innerExtra'] = innerExtra;
        }
        let json_str = JSON.stringify(req_json);
        this.trace(`C->S: replyInvite request= ${json_str}`);
        this.sendMsg(json_str);
    };
    statechangePublish(e) {
        this.trace(`[pushAndLive] publish statechangePublish = ${JSON.stringify(e)}`);
        let code = e.detail.code;
        let message = e.detail.message;
        switch (code) {
            case 1003:
                this.ReportClientEvent(322);
                break;
            case 1007:
                let extra = { streamId: this.publishStreamId }
                this.ReportClientEvent(503, extra);
                break;
            case -1301:
                this.ReportClientEvent(366);
                break;
            case -1302:
                this.ReportClientEvent(367)
                break;
            default:
                break;
        }
        if (code < 0) {
            wx.showModal({
                title: '系统提示',
                content: code + ': ' + message,
                confirmText: '确认',
                showCancel: false,
                success: (res) => { }
            })
        }
    }
    //发布网络变更监听
    pusherbindnetstatus(e) {
        this.trace(`[pushAndLive] publish pusherbindnetstatus ${JSON.stringify(e)}`);
        let pusher = {};
        pusher.id = e.currentTarget.id;
        pusher.videoBitrate = e.detail.info.videoBitrate;
        pusher.audioBitrate = e.detail.info.audioBitrate;
        pusher.videoFPS = parseInt(e.detail.info.videoFPS);
        pusher.videoWidth = e.detail.info.videoWidth;
        pusher.videoHeight = e.detail.info.videoHeight;
        return pusher
    }
    pusherbinderror(e) {
        const errcode = e.detail.errCode;
        const errmsg = e.detail.errMsg;
        this.trace(`[pushAndLive] publish pusherbinderror code:${errcode},msg:${errmsg}`);
        switch (errcode) {
            case 10001:
                this.ReportClientEvent(363)
                break;
            case 10002:
                this.ReportClientEvent(364);
            default:
                break;
        }
    }
    ReportClientEvent(code, extra) {
        if (!this.mtc) return
        let req_json = {};
        req_json['opcmd'] = 1403;
        req_json['requestId'] = this.getRquestId();
        req_json['sessionId'] = this.sessionId;
        if (this.connectionId) {
            req_json['connectionId'] = this.connectionId;
        }
        req_json['eventType'] = code;
        req_json['eventDesc'] = eventDesc[code] && eventDesc[code].desc;
        if (eventDesc[code].extra) {
            req_json["eventExtra"] = extra
        };
        req_json['clientTime'] = Date.now();
        const requestData = JSON.stringify(req_json);
        this.trace(`C-->S ReportClientEvent requestInfo = ${requestData}`);
        this.sendMsg(requestData);
    }
    //订阅拉流状态变更监听
    statechangeSubscribe(e) {
        this.trace(`[pushAndLive] subscribe statechangeSubscribe = ${JSON.stringify(e)}`);
        let code = e.detail.code;
        let feedId = e.currentTarget.dataset.feedid;
        let streamId = e.currentTarget.dataset.streamid;
        if (code == 2003) {
            this.ReportClientEvent(503, { feedId, streamId })
        }
    }
    //订阅拉流网络状态变更监听
    playerbindnetstatus(e) {
        this.trace(`[pushAndLive] subscribe playerbindnetstatus = ${JSON.stringify(e)}`);
        let obj = {};
        obj.id = e.currentTarget.id;
        obj.videoBitrate = e.detail.info.videoBitrate;
        obj.audioBitrate = e.detail.info.audioBitrate;
        obj.videoFPS = parseInt(e.detail.info.videoFPS);
        obj.videoWidth = e.detail.info.videoWidth;
        obj.videoHeight = e.detail.info.videoHeight;
        return obj
    }
    playerbinderror(e) {
        const errcode = e.detail.errCode;
        const errmsg = e.detail.errMsg;
        this.trace(`[pushAndLive] subscribe playerbinderror code:${errcode},msg:${errmsg}`);
        this.ReportClientEvent(errcode)
    }
    //离开房间
    LeaveRoom() {
        //取消发布
        // this.UnPublish();
        //取消订阅
        // this.UnSubscribe();
        //请求离开房间
        if (this.room_status == "close") {
            this.trace("room is already closed");
            return -1;
        }
        this.trace(`uid=${this.uid_} LeaveRoom`);
        this.leaveType = 1;
        //先发最后一次心跳,必选服务端心跳超时
        this.heartbeat();
        this.closeRoom(true);
        // this.Disconnect();
    }

    /* 通用功能接口 */
    /* 通用功能接口 */
    /* 通用功能接口 */
    //获取系统信息
    getSystemInfo() {
        const systemInfo = wx.getSystemInfoSync();
        this.trace(`systemInfo = ${JSON.stringify(systemInfo)}`);
        this.systemInfo = systemInfo;
        return systemInfo;
    }

    //获取rquestId
    getRquestId() {
        return ++this.requestId;
    }

    //获取uid
    getUid() {
        return `${new Date()}_wx`;
    }

    debug(text) {
        console.debug(`${this.gethhmmss()}: ${text}`);
    }

    trace(text) {
        console.log(`${this.gethhmmss()}: ${text}`);
    }

    warning(text) {
        console.log("%c" + `${this.gethhmmss()}: ${text}`, "color:red");
    }

    // 创建补0函数
    timeFormat(s) {
        return s < 10 ? '0' + s : s;
    }

    //获取当前时间戳
    gethhmmss() {
        let now = new Date();
        let year = now.getFullYear();
        let month = now.getMonth() + 1;
        let date = now.getDate();
        let hours = now.getHours();
        let minutes = now.getMinutes();
        let seconds = now.getSeconds();
        let milliseconds = now.getMilliseconds();
        return `[${this.timeFormat(year)}-${this.timeFormat(month)}-${this.timeFormat(date)} ${this.timeFormat(hours)}:${this.timeFormat(minutes)}:${this.timeFormat(seconds)}.${this.timeFormat(milliseconds)}]`;
    };

    //获取系统信息
    getARTVCUserInfo(extra) {
        let systemInfo = this.getSystemInfo();
        let ARTVCUserInfo = {};
        ARTVCUserInfo.clientType = 6;
        ARTVCUserInfo.os = systemInfo.system;
        ARTVCUserInfo.webrtcV = systemInfo.SDKVersion;
        ARTVCUserInfo.alipayV = systemInfo.version;
        ARTVCUserInfo.model = systemInfo.model;
        extra.ARTVCUserInfo = ARTVCUserInfo;
    }

    //检查系统版本
    checkSystemInfo(app) {
        wx.getSystemInfo({
            success: function (res) {
                log(`${JSON.stringify(res)}`);
                let sdkVersion = res.SDKVersion;
                let version_items = sdkVersion.split(".");
                let major_version = parseInt(version_items[0]);
                let minor_version = parseInt(version_items[1]);
                app.globalData.systemInfo = res;
                if (major_version <= 1 && minor_version < 7) {
                    wx.showModal({
                        title: '版本过低',
                        content: '微信版本过低，部分功能可能无法工作',
                        success: function (res) {
                            if (res.confirm) {
                                this.trace('用户点击确定');
                            } else if (res.cancel) {
                                this.trace('用户点击取消');
                            }
                        }
                    })
                }
            }
        })
    }

}