var mtcSdkVersion = "1.1.1";
export default class MtcHandler {

    constructor() {
        // 是否连接
        this.network = false;
        // 是否人为退出
        this._isClosed = false;
        this.socketTask = null;
        this.OnMtcOpen = null;
        this.OnMtcReOpen = null;
        this.OnMtcClose = null;
        this.OnMtcError = null;
        this.OnMtcReqSignInfo = null;
        this.OnMtcMessage = null;
        let now = Date.now();

    }

    //处理收到的mmessage
    onHandleMsg(evt) {
        let receive_msg = evt.data;
        let msg = JSON.parse(receive_msg);
        this.trace(`[mtc onReceiveMessage] recv msg=${receive_msg}`);

        // ###################### code == -103, 鉴权不通过 ######################
        if (msg.code == -103) {
            this.warning("[mtc onReceiveMessage] MTC auth failed!!!");
            // token鉴权失败，关闭SDK
            this.closeSdkPassively(20003, "token auth failed!");
            return;
        }

        // ###################### mtype = 201, 请求重传后，服务端确认包丢失 ######################
        if (msg.mtype == 201) {
            if (msg.code == -104) {
                var lossPushMsgId = JSON.parse(msg.body).lossPushMsgId;
                this.warning(`[mtc onReceiveMessage] ########## lossPushMsgId: ${lossPushMsgId}, ackPushMsgId: ${this.ackPushMsgId}`);
                if (lossPushMsgId <= this.newPushMsgId && lossPushMsgId > this.ackPushMsgId) {
                    this.ackPushMsgId = lossPushMsgId;
                }
                this.ask_repush_times = 0;
                this.ask_repush_last_Time = 0;
                // 处理包
                this.handleMsg();
            }

            // ###################### mtype = 203, 心跳ack ######################
        } else if (msg.mtype == 203) {
            this.hb_ok_ = true;
            if (this.hb_last_time > 0) {
                this.hb_rtt = new Date().getTime() - this.hb_last_time;
            }

            // ###################### mtype = 207, 初始化会话 ######################
        } else if (msg.mtype == 207) {
            if (msg.code == 200) {
                // 初始化成功
                this.init_ok = true;
                this.mtc_state = "running";
                this.token = msg.mtctoken;
                var initConfig = JSON.parse(msg.body);
                this.ackPushMsgId = initConfig.initAckPushMsgId;
                this.newPushMsgId = initConfig.initNewPushMsgId;
                this.trace(`[mtc onReceiveMessage] init OK! token=${this.token},ackPushMsgId=${this.ackPushMsgId},newPushMsgId=${this.newPushMsgId},time cost: ${new Date().getTime() - this.init_begin_time}ms`);
                //启动心跳
                if (this.heartbeat_timer_id == -1) {
                    let now = Date.now();
                    this.heartbeat_timer_id = setInterval(function () {
                        // wx.showToast({title: String(Date.now() - now)})
                        now = Date.now();
                        this.heartbeat();
                    }.bind(this), this.hb_interval);
                    this.trace(`[mtc onReceiveMessage] init OK! begin heartbeat timer, timerId = ${this.heartbeat_timer_id}`);
                }
                // 关闭init check timer
                this.resetInitMtc();
                // 通知上层业务
                this.innerOnMtcOpen();
            } else {
                this.closeSdkPassively(20004, "get token failed");
            }

            // ###################### mtype = 209, 正常业务处理 ######################
        } else if (msg.mtype == 209) {
            let pushMsgId = msg.mid;
            // 针对服务端网络抖动, pushMsgId出错, 直接吐数据给上层
            if (pushMsgId == -1 && msg.timestamp > this.lastRecvNewPushTime) {
                this.warning(`[mtc onReceiveMessage] ########## server pushMsgId occur something wrong [exception] : ${JSON.stringify(msg)}`);
                // 业务处理
                this.innerOnMtcMessage(msg.body);
                // 处理接下来的消息
                this.handleMsg();
                return;
            }

            // 检查pushMsgId合法性
            if (pushMsgId <= 0) {
                this.warning(`[mtc onReceiveMessage] ########## recv push msg error: invalid pushMsgId. msg: ${JSON.stringify(msg)}`);
                return;
            }
            // 返回ack
            this.sendAck(pushMsgId);

            // 检查重复包
            if (pushMsgId <= this.ackPushMsgId) {
                if (msg.timestamp > this.lastRecvNewPushTime) {
                    // 服务端pushMsgId数据丢失, 重置游标
                    this.warning(`[mtc onReceiveMessage] ########## server pushMsgId occur something wrong [loss]. ackPushMsgId: ${this.ackPushMsgId}, pushMsgId: ${pushMsgId}, newPushMsgId: ${this.newPushMsgId}`);
                    this.ackPushMsgId = pushMsgId - 1;
                    this.newPushMsgId = pushMsgId - 1;
                    this.packageMap = new Map();
                } else {
                    // 重复包不处理
                    this.warning(`[mtc onReceiveMessage] ########## recv duplicated push msg. ackPushMsgId: ${this.ackPushMsgId}, pushMsgId: ${pushMsgId}`);
                    return;
                }
            }
            // 跳包打日志
            if (pushMsgId > this.ackPushMsgId + 1) {
                this.warning(`[mtc onReceiveMessage] ########## recv jumping push msg. ackPushMsgId: ${this.ackPushMsgId}, pushMsgId: ${pushMsgId}`);
            }
            // 更新最新包
            if (this.newPushMsgId < pushMsgId) {
                this.newPushMsgId = pushMsgId;
                this.lastRecvNewPushTime = msg.timestamp;
            }

            // 缓存包
            this.packageMap.set(pushMsgId, msg.body);
            // 处理消息
            this.handleMsg();

            // ###################### mtype = 213, 恢复会话 ######################
        } else if (msg.mtype == 213) {
            if (msg.code == 200) {
                this.warning("[mtc onReceiveMessage] recover session success!");
                this.sendCacheMsg();
                this.innerOnMtcReOpen();
                if (this.heartbeat_timer_id == -1) {
                    this.heartbeat_timer_id = setInterval(function () {
                        this.heartbeat();
                    }.bind(this), this.hb_interval);
                    this.trace(`[mtc onReceiveMessage] init OK! begin heartbeat timer, timerId = ${this.heartbeat_timer_id}`);
                }
            } else {
                this.closeSdkPassively(20008, "recover session fail");
            }
        }

    }

    sendCacheMsg() {
        if (this.msgSendCacheIndex <= 0) {
            return;
        }
        this.trace(`[sendCacheMsg] num=${this.msgSendCacheIndex}`);
        for (var i = 0; i < this.msgSendCacheIndex; i++) {
            this.sendToMTC(this.msgSendCache.get(i));
            this.msgSendCache.delete(i);
        }
        this.msgSendCacheIndex = 0;
    };

    sendAck(pushMsgId) {
        var body = {
            "timestamp": new Date().getTime(),
            "pushMsgId": pushMsgId
        };
        var req = {
            "mtype": 205,
            "mid": this.getMid(),
            "mtctoken": this.token,
            "bizName": this.biz_name,
            "subBiz": this.sub_biz,
            "uid": this.uid_,
            "workspaceId": this.workspaceId,
            "body": body,
            "version": mtcSdkVersion
        };
        var reqStr = JSON.stringify(req);

        if (!this.sendToMTC(reqStr)) {
            this.warning(`[sendAck] pushMsgId = ${pushMsgId}, ack failed!`);
        }
    };

    handleMsg() {
        // 处理包
        this.handleReadyMsg();
        // 检查是否丢包
        while (this.newPushMsgId > this.ackPushMsgId) {
            // 请求重传次数未达到上限，请求重传
            if (this.ask_repush_times < this.ask_repush_times_max) {
                // 1s内只请求一次重传
                if (new Date().getTime() - this.ask_repush_last_Time > this.ask_repush_time_interval) {
                    // 首次请求重传延后1s
                    var repushMsgId = this.ackPushMsgId + 1;
                    if (this.ask_repush_times == 0) {
                        setTimeout(
                            () => {
                                this.askRepush(repushMsgId);
                            }, 1000);
                    } else {
                        this.askRepush(repushMsgId);
                    }
                    this.ask_repush_times++;
                    this.ask_repush_last_Time = new Date().getTime();
                }
                // 一旦发起重传请求，就等待服务端返回对应包，不再继续执行
                break;
            } else {
                // 超过重传次数，认为包已确认丢失
                this.trace(`[handleMsg] ########## lossPushMsgId: ${this.ackPushMsgId}, ackPushMsgId: ${this.ackPushMsgId + 1}`);
                this.ackPushMsgId++;
                this.ask_repush_times = 0;
                this.ask_repush_last_Time = 0;
                // 处理包
                this.handleReadyMsg();
            }
        }
    }

    askRepush(pushMsgId) {
        if (this.ackPushMsgId >= pushMsgId) {
            this.trace(`[askRepush] ${pushMsgId} is received!`);
            return;
        }
        this.trace(`[askRepush] ########## ask for repush. ackPushMsgId: ${this.ackPushMsgId}, repushMsgId: ${pushMsgId}, tryTimes: ${this.ask_repush_times}, repushLastTime: ${this.ask_repush_last_Time}`);
        var body = {
            "timestamp": new Date().getTime(),
            "startPushMsgId": pushMsgId
        };
        var req = {
            "mtype": 201,
            "mid": this.getMid(),
            "mtctoken": this.token,
            "bizName": this.biz_name,
            "subBiz": this.sub_biz,
            "uid": this.uid_,
            "workspaceId": this.workspaceId,
            "body": body,
            "version": mtcSdkVersion
        };

        var reqStr = JSON.stringify(req);

        if (this.sendToMTC(reqStr)) {
            this.trace(`[askRepush] askRepush ${pushMsgId}`);
        } else {
            this.warning(`[askRepush] askRepush ${pushMsgId} failed!`);
        }
    };

    handleReadyMsg() {
        // 处理包
        while (this.packageMap.has(this.ackPushMsgId + 1)) {
            this.ackPushMsgId++;
            this.ask_repush_times = 0;
            this.ask_repush_last_Time = 0;
            // 业务处理
            this.innerOnMtcMessage(this.packageMap.get(this.ackPushMsgId));
            // 更新packageMap
            this.packageMap.delete(this.ackPushMsgId);
        }
    };


    askRepush(pushMsgId) {
        if (this.ackPushMsgId >= pushMsgId) {
            this.trace(`[askRepush] ${pushMsgId} is received!`);
            return;
        }
        this.trace(`[askRepush] ########## ask for repush. ackPushMsgId: ${this.ackPushMsgId}, repushMsgId: ${pushMsgId}, tryTimes: ${this.ask_repush_times}, repushLastTime: ${this.ask_repush_last_Time}`);
        var body = {
            "timestamp": new Date().getTime(),
            "startPushMsgId": pushMsgId
        };
        var req = {
            "mtype": 201,
            "mid": this.getMid(),
            "mtctoken": this.token,
            "bizName": this.biz_name,
            "subBiz": this.sub_biz,
            "uid": this.uid_,
            "workspaceId": this.workspaceId,
            "body": body,
            "version": mtcSdkVersion
        };

        var reqStr = JSON.stringify(req);

        if (this.sendToMTC(reqStr)) {
            this.trace(`[askRepush] askRepush ${pushMsgId}`);
        } else {
            this.warning(`[askRepush] askRepush ${pushMsgId} failed!`);
        }
    };

    closeSdkGracefully() {
        this.warning("[close sdk] gracefully!");
        //主动关闭sdk,不可重连
        this.closeSdk(true);
    };

    heartbeat() {
        if (!this.network) {
            return;
        }
        // 首次心跳
        let now = new Date().getTime();
        if (this.hb_last_time == 0) {
            this.hb_last_time = now;
        }

        if (this.hb_ok_) {
            this.hb_last_time = now;
        }

        let body = {
            "timestamp": now,
            "source": this.source_,
            "rtt": this.hb_rtt
        };

        let req = {
            "mtype": 203,
            "mid": this.getMid(),
            "mtctoken": this.token,
            "bizName": this.biz_name,
            "subBiz": this.sub_biz,
            "uid": this.uid_,
            "workspaceId": this.workspaceId,
            "body": body,
            "version": mtcSdkVersion
        };
        let reqStr = JSON.stringify(req);
        if (this.sendToMTC(reqStr)) {
            this.hb_ok_ = false;
            this.trace(`[mtc heartbeat] heartbeat request=${reqStr}`);
        } else {
            this.warning("[mtc heartbeat] can't send heartbeat!");
        }
    };

    // 恢复会话
    recoverSession() {
        let body = {
            "newId": this.newPushMsgId,
            "ackId": this.ackPushMsgId,
            "source": this.source_
        };
        let req = {
            "mtype": 213,
            "mid": this.getMid(),
            "mtctoken": this.token,
            "bizName": this.biz_name,
            "subBiz": this.sub_biz,
            "uid": this.uid_,
            "workspaceId": this.workspaceId,
            "body": body,
            "version": mtcSdkVersion
        };
        let reqStr = JSON.stringify(req);
        this.trace(`[recoverSession] req: ${reqStr}`);
        if (this.sendToMTC(reqStr)) {
            this.trace("[recoverSession] recover session!");
        } else {
            this.warning("[recoverSession] recover session fail!");
        }
    };

    // 初始化会话
    initSession() {

        // 获取签名
        let sign = this.innerOnMtcReqSignInfo(this.biz_name, this.sub_biz, this.uid_, this.workspaceId);
        if (sign == null || sign == '') {
            this.closeSdkPassively(20005, "get sign failed!");
            return;
        }

        let body = {
            "sign": sign,
            "source": this.source_
        };

        let req = {
            "mtype": 207,
            "mid": this.getMid(),
            "bizName": this.biz_name,
            "subBiz": this.sub_biz,
            "uid": this.uid_,
            "body": body,
            "workspaceId": this.workspaceId,
            "version": mtcSdkVersion
        };
        var reqStr = JSON.stringify(req);

        this.trace(`[initSession] req: ${reqStr}`);

        if (this.sendToMTC(reqStr)) {
            this.trace("[initSession] init session!");
        } else {
            this.warning("[initSession] init session fail!");
        }
    };

    sendToMTC(req) {

        if (!this.network) {
            this.warning("[sendToMTC] network is unavailable!");
            return false;
        }
        try {
            this.sendMsgImp(req);
            return true;
        } catch (e) {
            this.warning(`[sendToMTC] ${e}`);
        }
        return false;

    };

    initCheckTimer() {
        // 初始化超时
        if (!this.init_ok) {
            this.warning("[initCheckTimer] init expired! close SDK!");
            //关闭sdk
            this.mtc_state = "close";
            // 清除timer
            this.resetInitMtc();
            // 停止心跳连接
            this.resetHeatBeat();
            if (this.network) {
                this.socketTask.close();
            }
            return;
        }
    }


    closeSdkPassively(code, msg) {
        this.warning("[close sdk] passively!");
        this.closeSdk(true);
        this.innerOnMtcError(code, msg);
    }

    closeSdk(isUerClose) {
        this.mtc_state = "close";
        // 清除timer
        this.resetInitMtc();
        // 停止心跳连接
        this.resetHeatBeat();
        if (isUerClose) {
            this.closeConnection();
        }
        //通知上层
        this.innerOnMtcClose();
    }

    trace(text) {
        console.log(`${this.gethhmmss()}: ${text}`);
    }

    warning(text) {
        console.log("%c" + `${this.gethhmmss()}: ${text}`, "color:red");
    }

    // 创建补0函数
    timeFormat(s) {
        return s < 10 ? '0' + s : s;
    }

    //获取当前时间戳
    gethhmmss() {
        let now = new Date();
        let year = now.getFullYear();
        let month = now.getMonth() + 1;
        let date = now.getDate();
        let hours = now.getHours();
        let minutes = now.getMinutes();
        let seconds = now.getSeconds();
        let milliseconds = now.getMilliseconds();
        return `[${this.timeFormat(year)}-${this.timeFormat(month)}-${this.timeFormat(date)} ${this.timeFormat(hours)}:${this.timeFormat(minutes)}:${this.timeFormat(seconds)}.${this.timeFormat(milliseconds)}]`;
    };

    /**
     * 创建WebSocket连接
     * @param options 建立连接时需要的配置信息，这里是传入的url，即你的服务端地址，端口号不是必需的。
     */
    createConnection({
        options
    }) {
        let _this = this;
        //检查网络
        wx.getNetworkType({
             success(result) {
                if (result.networkType != 'none') {
                    // 开始建立连接
                    _this.socketTask = wx.connectSocket({
                        url: options.url,
                        header: {
                            'content-type': 'application/json'
                        },
                        method: 'post',
                        timeout: 5000,
                        perMessageDeflate: true,
                        success: res => {
                            console.log(`连接成功 res=${JSON.stringify(res)}`);
                            if (_this.reconnectTimer) {
                              clearTimeout(_this.reconnectTimer);
                              _this.reconnectTimer = null;
                            }
                        },
                        fail: err => {
                            console.error(`连接失败 err=${err}`)
                        },
                        complete: res => {
                            console.log("create socket result");
                            if (_this.reconnectTimer) {
                              clearTimeout(_this.reconnectTimer);
                              _this.reconnectTimer = null;
                            }
                        }
                    })
                    _this.socketTask.onOpen((res) => {
                        _this.trace(`[onWsOpen] websocket is opened, state = ${_this.mtc_state}`);
                        _this.network = true;
                        if (!_this.init_ok) {
                            _this.initSession();
                        } else {
                            _this.recoverSession();
                        }
                    });
            
                    _this.socketTask.onError((res) => {
                        _this.trace("[onWsError] websocket get an error");
                        console.error(res);
                        _this.network = false;
                        // 停止心跳连接
                        _this.resetHeatBeat();
                        // 进行重连
                        if(!_this._isClosed) {
                            _this.reconnect();
                        }
                    })
            
                    _this.socketTask.onClose(function (res) {
                        _this.trace("[onWsClose] websocket is close");
                        _this.network = false;
                        // 停止心跳连接
                        _this.resetHeatBeat();
                        // 检测是否是用户自己退出小程序
                        if (!_this._isClosed) {
                            // 进行重连
                            _this.reconnect();
                        }
                    });
            
                    _this.socketTask.onMessage(
                        (evt) => {
                            _this.onHandleMsg(evt);
                        }
                    )
                } else {
                    console.log('网络已断开');
                    _this.network = false;
                    if (_this.reconnectTimer) {
                      clearTimeout(_this.reconnectTimer);
                      _this.reconnectTimer = null;
                    }
                    _this.innerOnMtcError(-10009, '网络错误');
                }
            },
            fail(res){
                console.log("get network type failed");
                if (_this.reconnectTimer) {
                  clearTimeout(_this.reconnectTimer);
                  _this.reconnectTimer = null;
                }
            }
        })
    }

    //获取mid
    getMid() {
        return ++this.mid_no;
    }

    //发送请求
    sendMsgImp(content) {
        this.socketTask.send({
            data: content,
            success: () => {
            },
            fail: () => {
            }

        })
    }

    //关闭websocket连接
    closeConnection() {
        this._isClosed = true;
        if (this.network) {
            this.socketTask.close();
        }
    }

    // 心跳重置
    resetHeatBeat() {
        if (this.heartbeat_timer_id != -1) {
            clearInterval(this.heartbeat_timer_id);
            this.heartbeat_timer_id = -1;
        }
        return this;
    }

    //重置初始化
    resetInitMtc() {
        if (this.init_check_timer_id != -1) {
            clearInterval(this.init_check_timer_id);
            this.init_check_timer_id = -1;
        }
        return this;
    }

    // ###################### 以下：接入方需要实现 ######################
    // ###################### 以下：接入方需要实现 ######################
    // ###################### 以下：接入方需要实现 ######################
    innerOnMtcOpen() {
        this.trace("[innerOnMtcOpen] mtc open");
        this.OnMtcOpen && this.OnMtcOpen();
    }

    innerOnMtcReOpen() {
        this.trace("[innerOnMtcReOpen] mtc reopen");
        this.OnMtcReOpen && this.OnMtcReOpen();
    }

    innerOnMtcClose() {
        this.trace("[innerOnMtcClose] sdk close");
        this.OnMtcClose && this.OnMtcClose();
    }

    innerOnMtcError(code, msg) {
        // this.warning(`[innerOnMtcError] init code = ${code}, msg = ${msg}`);
        this.OnMtcError && this.OnMtcError(code, msg);
    }

    innerOnMtcReqSignInfo(bizName, subBiz, uid, workspaceId) {
        // return 'signature';
        return this.OnMtcReqSignInfo && this.OnMtcReqSignInfo(bizName, subBiz, uid, workspaceId);
    }

    innerOnMtcMessage(msg) {
        // this.trace(`[innerOnMtcMessage] msg = ${JSON.stringify(msg)}`);
        this.OnMtcMessage && this.OnMtcMessage(msg);
    }

    init(config_param) {
        if (this.mtc_state == "running") {
            this.warning("[init] MTC_SDK is running!");
            return;
        }
        this.mtc_state = 'init';
        this.mid_no = 1;
        this.source_ = 'browse chrome';
        // 断网重连
        this.network = false;
        // 是否人为退出
        this._isClosed = false;
        // 请求token
        this.token = null;
        // 本地确认ack的包,用于检查跳包\重复包
        this.ackPushMsgId = 0;
        this.newPushMsgId = 0;
        this.lastRecvNewPushTime = 0;

        // 请求重传次数
        this.ask_repush_times = 0;
        this.ask_repush_times_max = 3;
        this.ask_repush_last_Time = 0;
        this.ask_repush_time_interval = 1000;

        this.packageMap = new Map();
        this.msgSendCache = new Map();
        this.msgSendCacheIndex = 0;

        // 心跳参数
        this.hb_interval = 10 * 1000;
        this.heartbeat_timer_id = -1;
        this.hb_ok_ = false;
        this.hb_rtt = 0;
        this.hb_last_time = 0;

        // 初始化
        this.init_check_timer_id = -1;
        this.init_interval = 5000;
        this.init_begin_time = 0;
        this.init_ok = false;

        // 配置参数
        this.workspaceId = config_param.workspaceId ? config_param.workspaceId : "default";
        // uid
        if (!config_param.uid) {
            this.closeSdkPassively(10001, 'invalid params');
            return;
        }
        this.uid_ = config_param.uid;

        // bizName
        if (!config_param.biz_name) {
            this.closeSdkPassively(10001, 'invalid params');
            return;
        }
        this.biz_name = config_param.biz_name;

        // subBiz
        if (!config_param.sub_biz) {
            this.closeSdkPassively(10001, 'invalid params');
            return;
        }
        this.sub_biz = config_param.sub_biz;

        // url
        if (!config_param.mtc_server_url) {
            this.closeSdkPassively(10001, 'invalid params');
            return;
        }
        this.mtc_server_url = config_param.mtc_server_url;
        // heartbeat(ms)
        if (config_param.hb_interval) {
            this.hb_interval = config_param.hb_interval;
            this.trace(`[init] #config# hb_interval: ${this.hb_interval}`);
        }

        // 请求重传次数上限
        if (config_param.ask_repush_times_max &&
            config_param.ask_repush_times_max < 5 && config_param.ask_repush_times_max > 0) {
            this.ask_repush_times_max = config_param.ask_repush_times_max;
            this.trace(`[init] #config# ask_repush_times_max: ${this.ask_repush_times_max}`);
        }

        // 请求重传间隔时间(ms)
        if (config_param.ask_repush_time_interval) {
            this.ask_repush_time_interval = config_param.ask_repush_time_interval;
            this.trace(`[init] #config# ask_repush_time_interval: ${this.ask_repush_time_interval}`);
        }

        this.resetHeatBeat();

        this.trace(`[mtc init] #config# uid= ${this.uid_},biz_name= ${this.biz_name},sub_biz= ${this.sub_biz},mtc_server_url= ${this.mtc_server_url},workspaceId= ${this.workspaceId}`);



        //检测网络变化  需要进行重连
        wx.onNetworkStatusChange(res => {
            this.trace(`当前网络状态=${res.isConnected},网络类型=${res.networkType}`);
            //网络关闭了才重连
            if (!this.network && !this._isClosed) {
                //进行重连
                this.reconnect();
            }
        })

        this.innerCreateConnection();
    }

    // 重连方法，会根据时间频率越来越慢
    reconnect() {
        if(this.reconnectTimer) {
            console.log("creating socket connection")
            return
        }
        let _this = this;
        this.reconnectTimer = setTimeout(() => {
            _this.createConnection({
                options: {
                    url: _this.mtc_server_url
                }
            });
        }, 3000);
    }

    innerCreateConnection() {
        if (!this.network) {
            this.init_begin_time = new Date().getTime();
            // 初始化检查timer
            if (this.init_check_timer_id == -1) {
                this.init_check_timer_id = setTimeout(function () {
                    this.initCheckTimer();
                }.bind(this), this.init_interval);
            }
            this.createConnection({
                options: {
                    url: this.mtc_server_url
                }
            });
        } else {
            console.log("您已经登录了");
        }
    }


    //主动关闭sdk
    close() {
        this.closeSdkGracefully();
    };


    //发送数据
    send(body, check) {
        if (this.mtc_state != "running") {
            this.warning(`[send] MTC_SDK is in wrong state: ${this.mtc_state}`);
            this.innerOnMtcError(30001, "send msg error");
            return -1;
        }
        var req = {
            "mtype": 209,
            "mid": this.getMid(),
            "mtctoken": this.token,
            "bizName": this.biz_name,
            "subBiz": this.sub_biz,
            "uid": this.uid_,
            "workspaceId": this.workspaceId,
            "body": body,
            "version": mtcSdkVersion
        };

        var reqStr = JSON.stringify(req);

        if (!this.network) {
            this.msgSendCache.set(this.msgSendCacheIndex, reqStr);
            this.msgSendCacheIndex++;
            this.innerOnMtcError(30001, "send msg error");
            return -1;
        }
        this.sendToMTC(reqStr)
        this.trace(`[mtc send] request= ${reqStr}`);
        return 0;
    };

}